#!/usr/bin/env python
import wx, os, ConfigParser, tarfile, shutil
from dialogs import CSDialog, IODialog, OKDialog

def init_dplug(dplug):
    dplug.plugins.project = Project(dplug)
    dplug.add_to("quit", "projects")
    dplug.add_cmd("project", "projects")

def trigger(trig, dplug):
    if trig == "quit":
        if dplug.plugins.project.status == "Enabled":
            dplug.plugins.project.close_project(dplug.plugins.project.projname)

def command(cmd, next, dplug):
    n = next.strip().split(" ")
    if n[0] == "create":    dplug.plugins.project.new_project()
    elif n[0] == "open" and len(n) == 2:    dplug.plugins.project.Open(n[1])
    elif n[0] == "close" or n[0] == "quit": dplug.plugins.project.close_project(dplug.plugins.project.projname)
    elif n[0] == "delete":  dplug.plugins.project.delete_project(dplug.plugins.project.projname)
    elif n[0] == "export":  dplug.plugins.project.export_project(dplug.plugins.project.projname)

class Project():
    """Class to manage Projects"""
    def __init__(self,dplug):
        self.dplug = dplug
        self.gui = dplug.main_class
        self.mode = "classic"
        self.status = "disabled"
        self.projname = ""
        self.projectsdir = os.path.join(self.dplug.plugindir, "Projects")
        if os.path.exists(self.projectsdir):
            pass
        else:
            os.mkdir(self.projectsdir)
            os.open(os.path.join(self.projectsdir, "projects.ded"), "w").close()
        self.projectsmenu = wx.Menu()
        self.menuid = wx.NewId()
        self.projectsmenu.Append(self.menuid, "New Project")
        openprojectmenu = wx.Menu()
        projects = self.GetProjects()
        for i in projects:
            x = openprojectmenu.Append(-1, i)
            self.gui.Bind(wx.EVT_MENU, self.__project_factory(i), x)
        self.projectsmenu.AppendMenu(-1, '&Open Project', openprojectmenu)
        self.gui.menu.Append(self.projectsmenu, "Projects")
        self.gui.Bind(wx.EVT_MENU, lambda event: self.new_project(), id=self.menuid)
    
    def __project_factory(self, i):
        """Switcher help function"""
        def _switcher(event):
            """helper function"""
            return self.Open(i)
        return _switcher
    
    def new_project(self):
        self.files_list = []
        
        self.projectpage = wx.Panel(self.gui.notebook)
        siz = wx.BoxSizer(wx.VERTICAL)
        self.name = wx.TextCtrl(self.projectpage)
        namesiz = wx.BoxSizer(wx.HORIZONTAL)
        namesiz.Add(wx.StaticText(self.projectpage, -1, "Project Name"))
        namesiz.Add(self.name)
        self.files = wx.ListBox(self.projectpage)
        buttons = wx.BoxSizer(wx.HORIZONTAL)
        self.add = wx.Button(self.projectpage, 0, "Add")
        self.remove = wx.Button(self.projectpage, 0, "Remove")
        self.create = wx.Button(self.projectpage, 0, "Create")
        self.folder = wx.Button(self.projectpage, 0, "Import Folder")
        self.tar = wx.Button(self.projectpage, 0, "Import Tar")
        buttons.AddSpacer((0, 0), 1, wx.EXPAND, 5)
        buttons.Add(self.add)
        buttons.Add(self.remove)
        buttons.Add(self.create)
        buttons.AddSpacer((0, 0), 1, wx.EXPAND, 5)
        buttons.Add(self.folder)
        buttons.Add(self.tar)
        siz.Add(namesiz, 0, wx.EXPAND)
        siz.Add(self.files, 1, wx.EXPAND)
        siz.Add(buttons, 0, wx.EXPAND)
        self.projectpage.SetSizer(siz)
        
        self.gui.add_tab("Project Overview", self.projectpage)
        self.gui.notebook.SetSelection(self.gui.notebook.GetPageCount()-1)
        
        self.add.Bind(wx.EVT_BUTTON, lambda event: self.add_file())
        self.remove.Bind(wx.EVT_BUTTON, lambda event: self.remove_file())
        self.create.Bind(wx.EVT_BUTTON, lambda event: self.create_project())
        self.folder.Bind(wx.EVT_BUTTON, lambda event: self.imp("folder"))
        self.tar.Bind(wx.EVT_BUTTON, lambda event: self.imp("tar"))
        
        self.gui.Layout()

    def add_file(self):
        dlg = wx.FileDialog(None, message="Select File", style=wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            self.files.Append(dlg.GetPath())
            self.files_list.append(dlg.GetPath())
        dlg.Destroy()
        self.check_commonprefix()
        
    def remove_file(self):
        self.files_list.remove(os.path.join(self.base, self.files.GetString(self.files.GetSelection())))
        self.files.Delete(self.files.GetSelection())
        self.check_commonprefix()
    
    def check_commonprefix(self):
        if len(self.files_list) > 1:
            self.base = os.path.commonprefix(self.files_list)
            self.files.Clear()
            self.files.Append("Common base path: %s" % self.base)
            for x in self.files_list:
                self.files.Append(x.split(self.base)[1])
        else:
            self.files.Clear()
            self.files.Append(self.files_list[0])

    def imp(self, type):
        if type == "folder":
            #path = IODialog("Importing Folder", "full path to folder (/home/user/development/)").GetValue()
            dlg = wx.DirDialog(self.projectpage, "Choose a directory:", style=wx.DD_DEFAULT_STYLE|wx.DD_DIR_MUST_EXIST)
            if dlg.ShowModal() == wx.ID_OK:
                self.mode = "classic"
                for f in os.walk(dlg.GetPath()):
                    for g in f[2]:
                        if not g.endswith(".pyc"):
                            self.files_list.append(os.path.join(f[0],g))
                            self.files.Append(os.path.join(f[0],g))
                self.check_commonprefix()
        elif type == "tar":
            #path = IODialog("Importing Tar", "full path to folder (/home/user/development/)").GetValue()
            dlg = wx.FileDialog(self.projectpage, message="Choose a file", wildcard="Tar files (*.tar; *.tar.gz)|*.tar;*.tar.gz",style=wx.OPEN|wx.CHANGE_DIR)
            if dlg.ShowModal() == wx.ID_OK:
                try:
                    tar = tarfile.open(dlg.GetPath())
                except:
                    print "O_o InVaLiD o_O"
                else:
                    self.mode = "tar"
                    tar.extractall(os.path.join(self.projectsdir, "taryard"))
                    tar.close()
                    for f in os.walk(os.path.join(self.projectsdir, "taryard")):
                        for g in f[2]:
                            self.files_list.append(os.path.join(f[0],g))
                            self.files.Append(os.path.join(f[0],g))
                    self.check_commonprefix()
                    self.tarpath = dlg.GetPath()
        else:
            print "error"

    def create_project(self):
        cf = os.path.join(self.projectsdir, "projects.ded")
        config = ConfigParser.ConfigParser()
        config.read(cf)
        if self.name.GetValue() in config.sections():
            OKDialog("Project Creation", "That Project name is already taken!")
            pass
        else:
            config.add_section(self.name.GetValue())
            config.set(self.name.GetValue(), "files", ",".join(self.files_list))
            
            with open(cf, "wb") as configf:
                config.write(configf)
            
            self.gui.notebook.RemovePage(self.gui.notebook.GetSelection())
            
            self.Open(self.name.GetValue())

    def Open(self,name):
        try:
            if CSDialog("Open Project", "Attention, this will close all open files").GetValue():
                tab = self.gui.get_active_tab()
                self.files = []
                for file in self.get_project_config(name, "files").strip().split(","):
                    self.files.append(file)
                #Save current opened files
                self.restore = []
                for tab in self.gui.tabs:
                    if tab.isFile and tab.path != None:
                        self.restore.append(tab.path)
                #Close all tabs
                i = 0
                while i < len(self.restore):
                    self.gui.notebook.RemovePage(0)
                    self.gui.tabs.pop(0)
                    i += 1
                self.gui.commandbar.SetValue("")
                #Open Project
                for file in self.files:
                    self.gui.editfl(file)
                projectmenu = wx.Menu()
                projectmenu.Append(50, "Close Project")
                projectmenu.Append(51, "Export Project")
                projectmenu.Append(52, "Delete Project")
                mid = -1
                for id, x in enumerate(self.gui.menu.GetMenus()):
                    if x[1] == "Projects":
                        self.gui.menu.Replace(id, projectmenu, name)
                self.gui.Bind(wx.EVT_MENU, lambda event:self.close_project(name), id=50)
                self.gui.Bind(wx.EVT_MENU, lambda event:self.export_project(name), id=51)
                self.gui.Bind(wx.EVT_MENU, lambda event:self.delete_project(name), id=52)
                self.status = "Enabled"
                self.projname = name
        except ConfigParser.NoSectionError:
            OKDialog("Project Open Error", "There is no project called %s!" % name)

    def close_project(self, name):
        #Remove project from notebook
        if self.mode == "tar":
            self.delete_project(name)
        else:
            project = []
            for tab in self.gui.tabs:
                if tab.isFile:
                    project.append(tab.path)
                self.gui.close_tab()
            #Restore previous files
            if len(self.restore)>0 and self.restore != ['None']:
                for file in self.restore:
                    self.gui.editfl(file)
            else:
                self.gui.new_file()
            self.projectsmenu = wx.Menu()
            self.menuid = wx.NewId()
            self.projectsmenu.Append(self.menuid, "New Project")
            openprojectmenu = wx.Menu()
            projects = self.GetProjects()
            for i in projects:
                x = openprojectmenu.Append(-1, i)
                self.gui.Bind(wx.EVT_MENU, self.__project_factory(i), x)
            self.projectsmenu.AppendMenu(-1, '&Open Project', openprojectmenu)                
            for id, x in enumerate(self.gui.menu.GetMenus()):
                if x[1] == name:
                    self.gui.menu.Replace(id, self.projectsmenu, "Projects")
            #Save changes to project (files added,removed)
            self.set_project_config(name, "files", ",".join(project))
            self.status = "Disabled"

    def export_project(self, name):
        tarname = IODialog("Export Project", "Name of the tarball? ([%s])" % name).GetValue()
        tar = tarfile.open(os.path.join(os.path.expanduser("~"),tarname + ".tar.gz"), "w:gz")
        tab = self.gui.get_active_tab()
        fls = []
        for tab in self.gui.tabs:
            fls.append(tab.path)
        os.chdir(os.path.commonprefix(fls))
        for file in fls:
            tar.add(file.split(os.path.dirname(os.path.commonprefix(fls)) + "/")[1])
        tar.close()

    def delete_project(self, name):
        if not self.mode == "tar" and CSDialog("Delete Project", "Are you sure?").GetValue():
            pass
        else:
            if self.mode == "tar":
                #TARFILES SHOULD BE CREATED AGAIN AND taryard should be cleared
                tar = tarfile.open(self.tarpath, "w:gz")
                tab = self.gui.get_active_tab()
                fls = []
                for tab in tabs:
                    fls.append(tab.path)
                os.chdir(os.path.commonprefix(fls))
                for file in fls:
                    tar.add(file.split(os.path.dirname(os.path.commonprefix(fls)) + "/")[1])
                tar.close()
                shutil.rmtree(os.path.join(self.projectsdir, "taryard"))
        self.mode = "classic" #fix to prevent loop
        self.close_project(name)
        cf = os.path.join(self.projectsdir, "projects.ded")
        config = ConfigParser.ConfigParser()
        config.read(cf)
        config.remove_section(name)

        with open(cf,"wb") as configf:
            config.write(configf)

       #Restore previous files
        for file in self.restore:
            self.gui.editfl(file)
        self.gui.menu.Remove(4)
        self.projectsmenu = wx.Menu()
        self.menuid = wx.NewId()
        self.projectsmenu.Append(self.menuid, "New Project")
        openprojectmenu = wx.Menu()
        projects = self.GetProjects()
        for id, x in enumerate(self.gui.menu.GetMenus()):
            if x[1] == name:
                self.gui.menu.Remove(id)
        for i in projects:
            x = openprojectmenu.Append(-1, i)
            self.gui.Bind(wx.EVT_MENU, self.__project_factory(i), x)
        self.projectsmenu.AppendMenu(-1, '&Open Project', openprojectmenu)
        self.gui.menu.Append(self.projectsmenu, "Projects")

    def GetProjects(self):
        cf = os.path.join(self.projectsdir, "projects.ded")
        config = ConfigParser.ConfigParser()
        config.read(cf)
        return config.sections()

    def set_project_config(self, section, name, value):
        """Changing the base configuration file,
        set_base_config(parent, section, name, value)"""
        cf = os.path.join(self.projectsdir, "projects.ded")
        config = ConfigParser.ConfigParser()
        config.read(cf)
        config.set(section, name, value)
        with open(cf,"wb") as configf:
            config.write(configf)

    def get_project_config(self, section, name):
        """Get value for a basic configuration setting
        get_base_config(parent, section, name)"""
        cf = os.path.join(self.projectsdir, "projects.ded")
        config = ConfigParser.ConfigParser()
        config.read(cf)
        return config.get(section, name)
