#!/usr/bin/env python
import wx, os, sys, inspect, subprocess, string
from wx import stc

def init_dplug(dplug):
    dplug.plugins.python = Python(dplug)
    dplug.add_to("save", "python")
    dplug.add_to("open", "python")
    dplug.add_to("sheetupdate", "python")
    dplug.add_to("sheetupdateup", "python")
    dplug.add_to("configsave", "python")
    dplug.add_to("configinit", "python")
    
    if dplug.plugin_config.get("python", "codecompletion") == "True":
        sys.path.insert(0, os.path.join(dplug.plugindir, "Python"))
        dplug.register("codecompletion")
        dplug.run("init_dplug", "codecompletion", dplug)

def trigger(trig, dplug, val = ""):
    if trig == "sheetupdate":
        return dplug.plugins.python.Update(val)
    elif trig == "sheetupdateup":
        return dplug.plugins.python.Update(val, up = True)
    elif trig == "configsave":
        dplug.plugins.python.config.Update()
    elif trig == "configinit":
        dplug.plugins.python.config.Init()
    elif trig == "save":
        dplug.plugins.python.save()
    elif trig == "open":
        dplug.plugins.python.pyc()

class Python():
    """Class to manage Projects"""
    def __init__(self,dplug):
        self.dplug = dplug
        self.gui = dplug.main_class
        
        _ = self.dplug.get_keybinding
        
        self.config = Config(self.gui.confpan, -1, i = self.dplug)

        #Conf
        self.dplug.plugin_config.add_section("python")
        if not self.dplug.plugin_config.get("python","autoindent"): self.dplug.plugin_config.set("python","autoindent","True")
        if not self.dplug.plugin_config.get("python","pyonlyindent"): self.dplug.plugin_config.set("python","pyonlyindent","True")
        if not self.dplug.plugin_config.get("python","analyze_method"): self.dplug.plugin_config.set("python","analyze_method","pylint")
        if not self.dplug.plugin_config.get("python","analyze_command"):
            self.dplug.plugin_config.set("python","analyze_command","pylint -d C0103 --reports=y")
        if not self.dplug.plugin_config.get("python","snippyts_underscore"): self.dplug.plugin_config.set("python","snippyts_underscore","False")
        if not self.dplug.plugin_config.get("python","snippyts_autoindent"): self.dplug.plugin_config.set("python","snippyts_autoindent","True")
        if not self.dplug.plugin_config.get("python","snippyts_pyc"): self.dplug.plugin_config.set("python","snippyts_pyc","True")
        
        tmpmenu = wx.Menu()
        aid = wx.NewId()
        rid = wx.NewId()
        rtid = wx.NewId()
        rsid = wx.NewId()
        iid = wx.NewId()
        aiid = wx.NewId()
        poid = wx.NewId()
        runmenu = wx.Menu()
        self.runcode = runmenu.Append(rid, "Normal\t" + _("python", "Run Code", "F5"))
        self.runtcode = runmenu.Append(rtid, "Print/Hold\t" + _("python", "Run Code (Print/Hold)", "shift+ctrl+r"))
        self.runscode = runmenu.Append(rsid, "Shell")
        tmpmenu.AppendMenu(-1, "Run Code", runmenu)
        self.analyzecode = tmpmenu.AppendCheckItem(aid, "Analyze code\t" + _("python", "Analyze Code", "shift+ctrl+a"))
        self.inspectcode = tmpmenu.AppendCheckItem(iid, "Inspect code\t" + _("python", "Inspect Code", "shift+ctrl+i"))
        self.gui.menu.Append(tmpmenu, "Python")
        
        self.gui.Bind(wx.EVT_MENU, lambda event: self.inspect_file(), id=iid)
        self.gui.Bind(wx.EVT_MENU, lambda event: self.run(), id=rid)
        self.gui.Bind(wx.EVT_MENU, lambda event: self.run("hold"), id=rtid)
        self.gui.Bind(wx.EVT_MENU, lambda event: self.run("shell"), id=rsid)
        self.gui.Bind(wx.EVT_MENU, lambda event: self.analyze(), id=aid)
        self.gui.Bind(wx.EVT_MENU, lambda event: self.ConfUpdate(), id=aiid)
        self.gui.Bind(wx.EVT_MENU, lambda event: self.ConfUpdate(), id=poid)
        
    def ConfUpdate(self):
        if self.autoindent.IsChecked(): self.dplug.plugin_config.set("python","autoindent","True")
        else: self.dplug.plugin_config.set("python","autoindent","False")
        if self.pyonlyindent.IsChecked(): self.dplug.plugin_config.set("python","pyonlyindent","True")
        else: self.dplug.plugin_config.set("python","pyonlyindent","False")

    def Update(self, val, up = False):
        evt, tab = val
        orig = tab.editor.GetCurrentPos()
        origsels, origsele = tab.editor.GetSelection()
        if evt.GetKeyCode() == 13 and not up:
            tab.editor.InsertText(tab.editor.GetCurrentPos(), "\n")
            tab.editor.LineDown()
            currentLine = tab.editor.GetCurrentLine()
            tab.editor.SetLineIndentation(currentLine, tab.editor.GetLineIndentation(currentLine - 1))
            tab.editor.LineEnd()
            return False
        elif evt.GetKeyCode() == 58 and up and self.dplug.plugin_config.get("python", "snippyts_autoindent") == "True":
            tab.editor.GotoLine(tab.editor.GetCurrentLine())
            tab.editor.SetSelection(tab.editor.GetCurrentPos(), tab.editor.GetLineEndPosition(tab.editor.GetCurrentLine()))
            found = False
            for x in ["def", "if", "elif", "else", "for", "while", "class"]:
                if x in tab.editor.GetSelectedText(): found = True
            if not found:
                tab.editor.GotoPos(orig)
                return True
            tab.editor.InsertText(tab.editor.GetCurrentPos(), "\n")
            tab.editor.LineDown()
            currentLine = tab.editor.GetCurrentLine()
            tab.editor.SetLineIndentation(currentLine, tab.editor.GetLineIndentation(currentLine - 1) + 4)  #Assuming your tab width is 4
            tab.editor.LineEnd()
            return True
        #Snippyts next release???
        """elif up and origsels == origsele:
            valid = string.letters + string.digits + "_"
            found = False
            lookup = [False, True, True, True]
            replace = ["def I():\n    ", "def %s(", "def %s(self, ", "def %s(self):\n    "]
            for n,x in enumerate(["plaindef", "def", "selfe", "selfn"]):
                tab.editor.SetSelection(tab.editor.GetCurrentPos() - len(x), tab.editor.GetCurrentPos())
                if tab.editor.GetSelectedText() == x and lookup[n]:
                    i = 1
                    while [k in valid for k in tab.editor.GetSelectedText()][0] == True:
                        tab.editor.SetSelection(tab.editor.GetCurrentPos() - len(x) - i, tab.editor.GetCurrentPos())
                        i+=1
                    tab.editor.SetSelection(tab.editor.GetCurrentPos() - len(x) - i + 2, tab.editor.GetCurrentPos())
                    tab.editor.ReplaceSelection(replace[n] % tab.editor.GetSelectedText()[:-len(x)])
                    tab.editor.GotoPos(tab.editor.GetLineEndPosition(tab.editor.GetCurrentLine()))
                    tab.editor.SetSelection(tab.editor.GetCurPos(), tab.editor.GetCurPos())
                    found = True
                elif tab.editor.GetSelectedText() == x and not lookup[n]:
                    tab.editor.ReplaceSelection(replace[n])
                    i = 0
                    while i < len(replace[n]) - replace[n].find("I"):
                        tab.editor.CmdKeyExecute(stc.STC_CMD_CHARLEFT)
                        i += 1
                    tab.editor.CmdKeyExecute(stc.STC_CMD_CHARRIGHTEXTEND)
                    tab.editor.SetSelection(tab.editor.GetCurPos(), tab.editor.GetCurPos())
                    found = True
                elif not found:
                    tab.editor.GotoPos(orig)
                    tab.editor.SetSelection(origsels, origsele)
            return True"""

    def run(self, mode="normal"):
        tab = self.gui.get_active_tab()
        if tab.isFile:
            if tab.name != "!New":
                current = os.getcwd()
                new = os.path.dirname(tab.path)
                os.chdir(new)
                if mode == "hold":
                    os.popen("xterm -hold -e python " + tab.name + " &")
                elif mode == "shell":
                    sys.path.append(os.getcwd())
                    self.dplug.main_class.console.interp.locals = __import__(tab.name[:-3]).__dict__
                    self.dplug.main_class.console.ClearAll()
                    self.dplug.main_class.console.run("print '%s is loaded into the shell'" % tab.name[:-3])
                else:
                    os.popen("xterm -e python " + tab.name + " &")
                os.chdir(current)
            else:
                if mode == "hold":
                    os.popen("xterm -hold -e python -c '" + os.path.basename(tab.editor.GetText()) + "' &")
                else:
                    os.popen("xterm -e python -c '" + os.path.basename(tab.editor.GetText()) + "' &")

    def analyze(self):
        """Run the code or Analyze the code"""
        tab = self.gui.get_active_tab()
        self.gui.command.SetValue("")
        self.gui.runpanel.SetValue("")
        if self.analyzecode.IsChecked():
            command = self.dplug.plugin_config.get("python", "analyze_command")
            command += " " + tab.path
            self.gui.command.SetValue(command)
            self.gui.runpanel.SetValue("Output will appear in some seconds..")
            self.gui.SetSize((1200, 600))
            self.gui.runpanel.Show()
            self.gui.command.Show()
            self.gui.Layout()
            value = os.popen(command).read()
            self.gui.runpanel.SetValue(value)
            #Markers
            x = 0
            for j in value.strip().split(":"):
                try:
                    line = int(j)-1
                    tab.editor.MarkerDefine(0, stc.STC_MARK_SHORTARROW, "#CC0000", "RED")
                    tab.editor.MarkerAdd(line, 0)
                    x=0
                except ValueError:
                    x+=1
        else:
            self.gui.SetSize((500, 600))
            self.gui.runpanel.Hide()
            self.gui.command.Hide()
            self.gui.Layout()
    
    def inspect_file(self):
        if self.inspectcode.IsChecked():
            self.root = self.gui.tree.AddRoot("Inspect")
            insp = Inspect(self)
            for i in insp.inspect():
                if i.startswith("__") and self.dplug.plugin_config.get("python", "snippyts_underscore") == "True": continue
                temp = self.gui.tree.AppendItem(self.root, i)
                for j in insp.inspect(i):
                    self.gui.tree.AppendItem(temp, j)
            self.gui.inspectpan.Show()
        else:
            self.gui.inspectpan.Hide()
            self.gui.tree.DeleteAllItems()
        self.gui.Layout()
    
    def python_file(self, file=""):
        """if no arguments passed the current opened file will be used,
        returns true or false"""
        tab = self.gui.get_active_tab()
        if file=="":
            temp = tab.name.strip().split(".")
        else:
            temp = file.strip().split(".")
        if temp[-1] == "py":    return True
        else:   return False
    
    def save(self):
        if self.python_file():
            tab = self.gui.get_active_tab()
            tab.editor.MarkerDeleteAll(0)
            for i in subprocess.Popen(["pyflakes", tab.path], shell=False, stdout=subprocess.PIPE,).communicate()[0].split("\n"):
                x = 0
                for j in i.strip().split(":"):
                    if (j == tab.name or j == tab.path) and x==0: #the filename
                        print "error in " + j
                    elif x==1: # the line number
                        line = int(j)-1
                        tab.editor.MarkerDefine(0, stc.STC_MARK_SHORTARROW, "#CC0000", "RED")
                        tab.editor.MarkerAdd(line, 0)
                        error = i.strip().split("'")[1]
                        l = tab.editor.GetText().strip().split("\n")[line]
                        tab.editor.IndicatorSetStyle(2, wx._stc.STC_INDIC_SQUIGGLE)
                        tab.editor.IndicatorSetForeground(2, wx.RED)
                        tab.editor.StartStyling(tab.editor.PositionFromLine(line) + len(l.split(error)[0]), wx._stc.STC_INDICS_MASK)
                        tab.editor.SetStyling(len(error), wx._stc.STC_INDIC2_MASK)
                        tab.editor.Colourise(0, -1)
                    else: #the actual error
                        x=-1
                    x+=1
    
    def pyc(self):
        tab = self.gui.get_active_tab()
        if tab.name.strip().split(".")[-1] == "pyc" and self.dplug.plugin_config.get("python", "snippyts_pyc") == "True":
            file = tab.path
            file = file[:-3] + "py"
            self.gui.close_tab(check=False)
            self.gui.editfl(file)
            self.gui.SetStatusText("INFO: Deditor opened the according .py file instead of the .pyc you selected, if you do want to open the .pyc file you need to uncheck this option @Python>Snippyts")

class Config(wx.Panel):
    def __init__(self, parent, sth, i):
        self.dplug = i
        wx.Panel.__init__(self, parent, sth)
        self.pythonbook = wx.Notebook(self)
        siz = wx.BoxSizer(wx.HORIZONTAL)
        
        #General Tab
        gen_panel = wx.Panel(self.pythonbook)
        gen_sizer = wx.GridSizer(2, 2, 0, 0)
        
        gen_sizer.Add(wx.StaticText(gen_panel, -1, "Activate codecompletion"))
        self.codecompletion = wx.CheckBox(gen_panel)
        gen_sizer.Add(self.codecompletion)
        
        gen_sizer.Add(wx.StaticText(gen_panel, -1, "Analyze command"))
        box = wx.BoxSizer(wx.VERTICAL)
        self.analyzecmd = wx.TextCtrl(gen_panel)
        box.Add(self.analyzecmd, 0, wx.EXPAND)
        gen_sizer.Add(box, -1, wx.EXPAND)
        
        gen_panel.SetSizer(gen_sizer)
        self.pythonbook.AddPage(gen_panel, "General")
        
        snippyts_panel = wx.Panel(self.pythonbook)
        snippyts_sizer = wx.GridSizer(7, 2, 0, 0)
        
        snippyts_sizer.Add(wx.StaticText(snippyts_panel, -1, "Hide __x__ elements from inspect"))
        self.underscore = wx.CheckBox(snippyts_panel)
        snippyts_sizer.Add(self.underscore)
        
        snippyts_sizer.Add(wx.StaticText(snippyts_panel, -1, "Auto Indent\ndef class if elif else for while"))
        self.autoindent = wx.CheckBox(snippyts_panel)
        snippyts_sizer.Add(self.autoindent)
        
        snippyts_sizer.Add(wx.StaticText(snippyts_panel, -1, "When you open a .pyc file close it\nand open the according .py file"))
        self.pyc = wx.CheckBox(snippyts_panel)
        snippyts_sizer.Add(self.pyc)
        
        """snippyts_sizer.Add(wx.StaticText(snippyts_panel, -1, "The following snippyts are\nrewrite options"))
        snippyts_sizer.Add(wx.StaticText(snippyts_panel, -1, "Input > Output,\nI: your carret"))
        
        snippyts_sizer.Add(wx.StaticText(snippyts_panel, -1, "testdef > def test(I"))
        self.testdef = wx.CheckBox(snippyts_panel)
        self.testdef.SetValue(1)
        self.testdef.Disable()
        snippyts_sizer.Add(self.testdef)
        
        snippyts_sizer.Add(wx.StaticText(snippyts_panel, -1, "testselfe > def test(self, I"))
        self.testselfe = wx.CheckBox(snippyts_panel)
        self.testselfe.SetValue(1)
        self.testselfe.Disable()
        snippyts_sizer.Add(self.testselfe)
        
        snippyts_sizer.Add(wx.StaticText(snippyts_panel, -1, "testselfn > def test(self):\n    I"))
        self.plaindef = wx.CheckBox(snippyts_panel)
        self.plaindef.SetValue(1)
        self.plaindef.Disable()
        snippyts_sizer.Add(self.plaindef)
        
        snippyts_sizer.Add(wx.StaticText(snippyts_panel, -1, "plaindef > def I():\n    "))
        self.testselfn = wx.CheckBox(snippyts_panel)
        self.testselfn.SetValue(1)
        self.testselfn.Disable()
        snippyts_sizer.Add(self.testselfn)"""
        
        snippyts_panel.SetSizer(snippyts_sizer)
        
        self.pythonbook.AddPage(snippyts_panel, "Snippyts")
        
        siz.Add(self.pythonbook, -1, wx.EXPAND)
        self.SetSizer(siz)
        
        self.button_name = "Python"

        parent.plugin_add(self)
        
    def Init(self):
        #General
        self.analyzecmd.SetValue(self.dplug.plugin_config.get("python", "analyze_command"))
        if self.dplug.plugin_config.get("python", "codecompletion") == "True":
            self.codecompletion.SetValue(1)
        else:
            self.codecompletion.SetValue(0)

        #Snippyts
        if self.dplug.plugin_config.get("python", "snippyts_underscore") == "True":
            self.underscore.SetValue(1)
        else:   self.underscore.SetValue(0)
        if self.dplug.plugin_config.get("python", "snippyts_autoindent") == "True":
            self.autoindent.SetValue(1)
        else:   self.autoindent.SetValue(0)
        if self.dplug.plugin_config.get("python","snippyts_pyc") == "True":
            self.pyc.SetValue(1)
        else:   self.pyc.SetValue(0)

    def Update(self):
        if self.codecompletion.GetValue() == 1: self.dplug.plugin_config.set("python", "codecompletion", "True")
        else: self.dplug.plugin_config.set("python", "codecompletion", "False")
        self.dplug.plugin_config.set("python", "analyze_command", self.analyzecmd.GetValue())
        if self.underscore.GetValue() == 1: self.dplug.plugin_config.set("python", "snippyts_underscore", "True")
        else:   self.dplug.plugin_config.set("python", "snippyts_underscore", "False")
        if self.autoindent.GetValue() == 1: self.dplug.plugin_config.set("python", "snippyts_autoindent", "True")
        else:   self.dplug.plugin_config.set("python", "snippyts_autoindent", "False")
        if self.pyc.GetValue() == 1: self.dplug.plugin_config.set("python", "snippyts_pyc", "True")
        else:   self.dplug.plugin_config.set("python", "snippyts_pyc", "False")

class Inspect():
    def __init__(self, env):
        self.dplug = env.dplug
        self.gui = env.gui
        self.tab = self.gui.get_active_tab()
        self.result = self.tab.name
    
    def inspect(self, txt=""):
        if txt=="":
            obj = self.get_object()
        else:
            obj = self.get_object(root = False, obj = txt)
        try:
            return obj.func_code.co_varnames
        except AttributeError:
            j = []
            for i in inspect.getmembers(obj):
                j.append(i[0])
            return j
    
    def get_object(self, root = True, obj=""):
        try:
            x = sys.modules[self.tab.name[:-3]]
        except KeyError:
            return False
        if root:
            return x
        elif obj != "":
            return getattr(x, obj)
