#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Deditor, Advanced text-editor for python on linux
Author: Kruptein (Darragh van Tichelen)
email:  kruptein@gmail.com"""

import wx, os, datetime, sys, ConfigParser, wx.aui, wx.lib.scrolledpanel
from deditorstc import DemoCodeEditor
from wx.py.shell import Shell
from com import Com
from Plugins.dialogs import CSDialog, IODialog, OKDialog
from TextCtrlAutoComplete import TextCtrlAutoComplete as TCAC

#GLOBALS
lastsearch = 0

def trigger(trig, dplug, val):
    if trig=="sheetupdateup":
        dplug.main_class.check_filechanged()

def command(cmd, next, dplug):
    if cmd == "quit":   dplug.main_class.quit()
    elif cmd == "open":
        if next == "":
            dplug.main_class.open_file()
        else:
            dplug.main_class.editfl(next.strip())
    elif cmd == "close":
        dplug.main_class.close_tab(next)
    elif cmd == "!New":
        dplug.main_class.new_file(path=next)
    elif cmd == "config":
        if next == "":
            dplug.main_class.config()
        elif next == "show":
            dplug.main_class.confpan.Init()
            dplug.main_class.HidePluginPan()
            dplug.main_class.notebook.Hide()
            dplug.main_class.confpan.Show()
            dplug.main_class.Layout()
        elif next == "hide":
            dplug.main_class.notebook.Show()
            dplug.main_class.confpan.Hide()
            dplug.main_class.Layout()
    elif cmd == "shell":
        if next == "":
            dplug.main_class.sh()
        elif next == "show":
            dplug.main_class.console.Show()
            dplug.main_class.Layout()
        elif next == "hide":
            dplug.main_class.console.Hide()
            dplug.main_class.Layout()
    elif cmd == "version":
        OKDialog("Version Information", "Deditor version %s" \
            % dplug.base_config.get("general","version"))
    elif cmd == "update":
        dplug.update()
    elif cmd == "find":
        dplug.main_class.find()
        if next != "":
            dplug.main_class.bartxt.SetValue(next)
            dplug.main_class.findbar()
    elif cmd == "syntax":
        if next == "":
            OKDialog("Syntax Command", "Missing Argument [Language]")
        else:
            dplug.main_class.lexer(next.upper())
    elif cmd == "size":
        w,h = next.strip().split("x")
        dplug.main_class.SetSize((int(w), int(h)))
        dplug.base_config.set("general","size",next)

class DEDITOR(wx.Frame):
    """The main instance"""
    def __init__(self, dplug):
        self.dplug = dplug
    
    def create(self, parent, id, title, pos, size, style):
        wx.Frame.__init__(self, parent, id, title, pos, size, style)
        self.notebook = wx.aui.AuiNotebook(self, -1)
        self.commandbar = TCAC(self, style=wx.TE_PROCESS_ENTER, choices=["test"])  #top textctrl which if enabled handles commands
        self.commandbar.Bind(wx.EVT_KEY_DOWN, self.filecompletion)
        self.console = Shell(self)  #It's the window on the bottom of the screen (if enabled)
        self.console.interp.locals = self.dplug.__dict__
        self.runpanel = wx.TextCtrl(self, -1,
            style=wx.TE_MULTILINE|wx.TE_READONLY)  #
        self.command = wx.TextCtrl(self, -1, style=wx.TE_READONLY)
        self.tabs = []
        
        #Keys Hack
        _ = self.dplug.get_keybinding
        
        self.menu = wx.MenuBar()
        filemenu = wx.Menu()
        filemenu.Append(1, "&New\t" + _("main","New File", "ctrl+n"))
        filemenu.Append(2, "&Open\t" + _("main","Open File", "ctrl+o"))
        filemenu.Append(3, "&Save\t" + _("main","Save File", "ctrl+s"))
        filemenu.Append(4, "&Save File As\t" + _("main","Save File As", "ctrl+shift+s"))
        filemenu.AppendSeparator()
        filemenu.Append(22, "&Reload\t" + _("main","Reload File", "ctrl+r"))
        filemenu.AppendSeparator()
        filemenu.Append(23, "&Close Tab\t" + _("main","Close File", "ctrl+w"))
        filemenu.Append(5, "&Quit\t" + _("main","Quit Deditor", "ctrl+q"))
        self.menu.Append(filemenu, "File")
        viewmenu = wx.Menu()
        self.shellview = viewmenu.AppendCheckItem(6, "Python Shell\t" + _("main","Show/Hide Python Shell", "shift+ctrl+p"))
        self.confview = viewmenu.AppendCheckItem(7, "Configuration\t" + _("main","Show/Hide Configuration", "shift+ctrl+c"))
        self.syntaxmenu = wx.Menu()
        self.proglangs = ['ADA', 'APDL', 'ASM', 'ASN1', 'AU3', 'AUTOMATIC', 'AVE', 'BAAN', 'BASH', 'BATCH',
            'BLITZBASIC', 'BULLANT', 'CAML', 'CLW', 'CLWNOCASE', 'CONF', 'CONTAINER', 'CPP',
            'CPPNOCASE', 'CSOUND', 'CSS', 'DIFF', 'EIFFEL', 'EIFFELKW', 'ERLANG', 'ERRORLIST',
            'ESCRIPT', 'F77', 'FLAGSHIP', 'FORTH', 'FORTRAN', 'FREEBASIC', 'GUI4CLI', 'HASKELL',
            'HTML', 'INNOSETUP', 'KIX', 'LATEX', 'LISP', 'LOT', 'LOUT', 'LUA', 'MAKEFILE', 'MATLAB',
            'METAPOST', 'MMIXAL', 'MSSQL', 'NNCRONTAB', 'NSIS', 'NULL', 'OCTAVE', 'OPAL', 'PASCAL',
            'PERL', 'PHPSCRIPT', 'POV', 'POWERBASIC', 'PROPERTIES', 'PS', 'PUREBASIC', 'PYTHON',
            'REBOL', 'RUBY', 'SCRIPTOL', 'SMALLTALK', 'SPECMAN', 'SPICE', 'SQL', 'TADS3', 'TCL',
            'TEX', 'VB', 'VBSCRIPT', 'VERILOG', 'VHDL', 'XCODE', 'XML', 'YAML']
        for i in self.proglangs:
            x = self.syntaxmenu.AppendRadioItem(-1, i)
            self.Bind(wx.EVT_MENU, self.__switcher_factory(i), x)
        viewmenu.AppendMenu(-1, '&Syntax', self.syntaxmenu)
        self.menu.Append(viewmenu, "View")
        findmenu = wx.Menu()
        findmenu.Append(10, "Find\t" + _("main","Find Text", "ctrl+f"))
        self.findmode = wx.Menu()
        for i in ["Normal", "WholeWord", "MatchCase", "WordStart", "Regex"]:
            x = self.findmode.Append(-1, i)
            self.Bind(wx.EVT_MENU, self.__findmode_factory(i), x)
        findmenu.AppendMenu(-1, "Find Mode", self.findmode)
        findmenu.Append(11, "Goto Line\t" + _("main","Goto Line Number", "ctrl+l"))
        self.menu.Append(findmenu, "Find")
        self.SetMenuBar(self.menu)

        self.deditor_statusbar = self.CreateStatusBar(1, 0)

        self.SetTitle("DEDITOR")
        _icon = wx.EmptyIcon()
        _icon.CopyFromBitmap(wx.Bitmap(os.path.join(os.path.dirname(__file__),"logo.gif"), wx.BITMAP_TYPE_ANY))
        self.SetIcon(_icon)
        self.deditor_statusbar.SetStatusWidths([-1])
        deditor_statusbar_fields = ["Deditor"]
        for i in range(len(deditor_statusbar_fields)):
            self.deditor_statusbar.SetStatusText(deditor_statusbar_fields[i], i)
        
        ###Runpanel###
        self.runpan = wx.BoxSizer(wx.VERTICAL)
        self.runpan.Add(self.command, 0, wx.EXPAND)
        self.runpan.Add(self.runpanel, 75, wx.EXPAND)

        ###InspectPan###
        self.inspectpan = wx.Panel(self)
        self.inspectsiz = wx.BoxSizer(wx.VERTICAL)
        self.tree = wx.TreeCtrl(self.inspectpan, style = wx.TR_HIDE_ROOT|wx.TR_HAS_BUTTONS)
        self.inspectsiz.Add(self.tree, -1, wx.EXPAND)
        self.inspectsiz.Fit(self.inspectpan)
        self.inspectpan.SetSizer(self.inspectsiz)

        ###Configuration###
        self.confpan = Config(self.dplug)

        ###Main###
        self.toolbar = wx.Panel(self)
        self.toolbars = wx.BoxSizer(wx.HORIZONTAL)
        self.toolbar.SetSizer(self.toolbars)
        self.bartxt = wx.TextCtrl(self.toolbar)
        self.toolbars.Add(self.bartxt, -1, wx.EXPAND)
        #self.findbut = wx.Button(self.toolbar, wx.ID_FIND)
        #self.toolbars.Add(self.findbut)
        self.findpr = wx.BitmapButton(self.toolbar, -1, wx.Bitmap(os.path.join(self.dplug.directory,  "icons",  "up.png")))
        self.toolbars.Add(self.findpr)
        self.findnx = wx.BitmapButton(self.toolbar, -1, wx.Bitmap(os.path.join(self.dplug.directory,  "icons",  "down.png")))
        self.toolbars.Add(self.findnx)
        self.qtb = wx.BitmapButton(self.toolbar, -1, wx.Bitmap(os.path.join(self.dplug.directory,  "icons",  "close.png")))
        self.toolbars.Add(self.qtb)
        
        self.pluginpan = wx.Panel(self)
        self.pluginsiz = wx.BoxSizer(wx.VERTICAL)
        self.pluginpan.SetSizer(self.pluginsiz)
        self.pluginpanels = []
        
        extrabar = wx.BoxSizer(wx.HORIZONTAL)
        self.open_file_button = wx.BitmapButton(self, -1, wx.Bitmap(os.path.join(self.dplug.directory,  "icons",  "open.png")))
        self.save_file_button = wx.BitmapButton(self, -1, wx.Bitmap(os.path.join(self.dplug.directory,  "icons",  "save.png")))
        if self.dplug.base_config.get("general", "design") == "extend":
            extrabar.Add(self.open_file_button)
            extrabar.Add(self.save_file_button)
            self.Bind(wx.EVT_BUTTON, lambda event: self.new_file(), self.open_file_button)
            self.Bind(wx.EVT_BUTTON, lambda event: self.save_file(), self.save_file_button)
        extrabar.Add(self.commandbar, -1, wx.EXPAND)

        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_3 = wx.BoxSizer(wx.HORIZONTAL)
        self.basic_sizer = wx.BoxSizer(wx.VERTICAL)
        self.basic_sizer.Add(self.notebook, 75, wx.EXPAND, 0)
        self.basic_sizer.Add(self.pluginpan, 75, wx.EXPAND, 0)
        self.basic_sizer.Add(self.toolbar, 0, wx.EXPAND, 0)
        self.basic_sizer.Add(self.console, 15, wx.EXPAND, 0)
        self.basic_sizer.Add(extrabar, 0, wx.EXPAND, 0)
        sizer_3.Add(self.inspectpan, 15, wx.EXPAND, 0)
        sizer_3.Add(self.basic_sizer, 50, wx.EXPAND, 0)
        sizer_3.Add(self.runpan, 50, wx.EXPAND, 0)
        sizer_1.Add(sizer_3, 1, wx.EXPAND, 0)
        self.SetSizer(sizer_1)
        sizer_1.Fit(self)
        self.inspectpan.Hide()
        self.runpanel.Hide()
        self.command.Hide()
        self.toolbar.Hide()
        self.pluginpan.Hide()
        self.Layout()
        try:
            x = self.dplug.base_config.get("general","size")
            w, h = x.strip().split("x")
        except ConfigParser.NoOptionError:
            w, h = (500, 600)
            
        self.SetSize((int(w), int(h)))

        self.Bind(wx.EVT_MENU, lambda event: self.new_file(), id=1)
        self.Bind(wx.EVT_MENU, lambda event: self.open_file(), id=2)
        self.Bind(wx.EVT_MENU, lambda event: self.save_file(), id=3)
        self.Bind(wx.EVT_MENU, lambda event: self.saveas(), id=4)
        self.Bind(wx.EVT_MENU, lambda event: self.reload_file(), id=22)
        self.Bind(wx.EVT_MENU, lambda event: self.close_tab(), id=23)
        self.Bind(wx.EVT_MENU, lambda event: self.quit(), id=5)
        self.Bind(wx.EVT_MENU, lambda event: self.sh(), id=6)
        self.Bind(wx.EVT_MENU, lambda event: self.config(), id=7)
        self.Bind(wx.EVT_MENU, lambda event: self.find(), id=10)
        self.Bind(wx.EVT_MENU, lambda event: self.find(), id=10)
        self.Bind(wx.EVT_MENU, lambda event: self.locate_line(), id=11)
        self.Bind(wx.aui.EVT_AUINOTEBOOK_PAGE_CLOSE, self.pclose)
        self.Bind(wx.aui.EVT_AUINOTEBOOK_PAGE_CLOSED, self.pclosed)
        self.Bind(wx.EVT_BUTTON, lambda event: self.cfindbar(), self.qtb)
        self.Bind(wx.EVT_BUTTON, lambda event: self.findprev(), self.findpr)
        self.Bind(wx.EVT_BUTTON, lambda event: self.findnext(), self.findnx)
        self.commandbar.Bind(wx.EVT_ENTER_WINDOW, lambda event: self.cmd_focus("set"))
        self.commandbar.Bind(wx.EVT_LEAVE_WINDOW, lambda event: self.cmd_focus("kill"))
        self.Bind(wx.EVT_CLOSE, lambda event: self.quit())
        
        self.initialize()

    def __switcher_factory(self, i):
        """Switcher help function"""
        def _switcher(event):
            """helper function"""
            return self.lexer(i)
        return _switcher
    
    def __findmode_factory(self, i):
        """Switcher help function"""
        def _switcher(event):
            """helper function"""
            self.mode = i
        return _switcher
        
    def initialize(self):
        """Initialize all the defaults and run the startup functions"""

        #Commands
        self.dplug.setup_cmd(self.commandbar)
        self.dplug.add_cmd("quit", "deditor")
        self.dplug.add_cmd("open", "deditor")
        self.dplug.add_cmd("close", "deditor")
        self.dplug.add_cmd("!New", "deditor")
        self.dplug.add_cmd("config", "deditor")
        self.dplug.add_cmd("shell", "deditor")
        self.dplug.add_cmd("version", "deditor")
        self.dplug.add_cmd("update", "deditor")
        self.dplug.add_cmd("find", "deditor")
        self.dplug.add_cmd("syntax", "deditor")
        self.dplug.add_cmd("size", "deditor")

        #Basic Triggers
        self.dplug.create("!New")
        self.dplug.create("open")
        self.dplug.create("save")
        self.dplug.create("quit")
        self.dplug.add_to("sheetupdateup", "deditor")
        
        self.shellview.Check()
        self.filedir = self.dplug.directory
        #Startup Conf
        self.extensions = [["PYTHON", "py"], ["HTML", "htm", "html", "xhtml"], ["CSS", "css"], ["ERLANG", "yaws"], ["PERL", "pl"], ["PHPSCRIPT", "php", "php4", "php3", "phtml"], ["RUBY", "rb", "rhtml"], ["XML", "xml", "rss", "svg"], ["ADA", "ada"]] #extend
        method = self.dplug.base_config.get("startup", "method")
        opn = self.dplug.base_config.get("startup", "open")
        focus = int(self.dplug.base_config.get("startup", "focus"))
        if focus < 0: focus = 0
        if method == "restore":
            files = opn.strip().split(",")
            if files[0] != "":
                for fl in files:
                    try:
                        self.editfl(fl)
                    except IOError:
                        pass
                self.notebook.SetSelection(int(focus))
            else:
                self.add_file_tab("!New")
        else:
            self.add_file_tab("!New")
        if len(self.tabs) == 0: self.add_file_tab("!New")
        if self.dplug.base_config.get("startup", "python") == "True":
            self.shellview.Check(1)
        else:
            self.shellview.Check(0)
        self.sh()
        #Opening Conf (Passed args)
        if len(sys.argv) >= 2:
            for i in sys.argv[1:]:
                self.editfl(i)
        #Findmenu
        self.mode = "Normal"
    
    def cmd_focus(self, focus):
        if focus == "kill":
            self.commandbar.SetSelection(0,0)
        else:
            self.commandbar.SetFocus()
            self.commandbar.SelectAll()

    #PluginPan Related Methods
    
    def AddpluginPan(self, pan):
        self.pluginsiz.Add(pan, -1, wx.EXPAND)
        self.pluginpanels.append(pan)
    
    def DeletePluginPan(self, pan):
        self.pluginsiz.Remove(pan)
        self.pluginpanels.remove(pan)
        self.notebook.Show()
    
    def ClearPluginPan(self):
        for i in self.pluginpanels:
            i.Hide()
    
    def ShowPluginPan(self):
        self.notebook.Hide()
        self.confpan.Hide()
        self.pluginpan.Show()
    
    def HidePluginPan(self):
        self.pluginpan.Hide()
        self.notebook.Show()

    #Notebook related Methods
    
    def pclose(self, event):
        """ this event is launched when the page is closing, reference to page still possible """
        event.Veto()
        tab = self.tabs[event.GetSelection()]
        if tab.name == "Configuration" and self.confview.IsChecked():
            self.confview.Check(0)
        self.tabs.remove(tab)
        self.notebook.RemovePage(event.GetSelection())
        if len(self.tabs) == 0:
            self.new_file()

    def pclosed(self, event):
        """ this event is launched when the page is closed, reference to page NOT possible """
        print "pclosed"

    def is_file_opened(self, fl):
        for tab in self.tabs:
            if tab.path == fl:
                return True
        return False

    def editfl(self, fl):
        """Print the content of the 'fl' file to a new tab"""
        current_tab = self.get_active_tab()
        page = self.notebook.GetSelection()
        if self.is_file_opened(fl) == True:
            return
        try:
            f = open(fl, 'r')
            txt = ""
            for line in f.readlines():
                txt = txt + line
            f.close()
            if current_tab.isFile:
                if current_tab.name == "!New" and current_tab.editor.GetText() == "":
                    self.tabs.pop(page)
                    self.notebook.RemovePage(page)
            self.add_file_tab(os.path.basename(fl), fl)
            #Set Page to new Tab
            self.notebook.SetSelection(self.notebook.GetPageCount()-1)
            page = self.notebook.GetSelection()
            tab = self.tabs[page]
            tab.editor.SetValue(txt)
            self.notebook.SetSelection(page)
            lang = self.get_proglang(tab.path)
            if lang == None:    self.syntaxmenu.Check(135 + self.proglangs.index(self.dplug.base_config.get("startup", "proglang")), True)
            else:
                self.syntaxmenu.Check(135 + self.proglangs.index(lang), True)
                self.lexer(lang)
            self.dplug.trigger_update("open")
        except IOError:
            pass
    
    def get_proglang(self, file):
        try:
            for i in self.extensions:
                if "." in file and os.path.split(file)[1].strip().split(".")[1] in i:
                    return i[0]
        except IndexError:
            return "PYTHON"

    def filecompletion(self, event):
        keycode = event.GetKeyCode()
        if keycode == wx.WXK_TAB:
            dr,nxt = "/".join(self.commandbar.GetValue().split("/")[0:-1]),self.commandbar.GetValue().split("/")[-1::][0]
            for x in os.listdir(dr):
                if x.startswith(nxt):
                    self.commandbar.SetValue(os.path.join(dr,x))
                    self.commandbar.SetInsertionPointEnd()
        elif keycode == wx.WXK_RETURN:
            found = self.dplug.evaluate()
            if not found:
                if os.path.isfile(self.commandbar.GetValue()):
                    self.editfl(self.commandbar.GetValue())
                else:
                    self.new_file(path=self.commandbar.GetValue())
        event.Skip()
    
    def check_filechanged(self):
        tab = self.get_active_tab()
        page = self.notebook.GetSelection()
        if tab.path != None and tab.name != "!New":
            f = open(tab.path, 'r')
            txt = ""
            for line in f.readlines():
                txt = txt + line
            f.close()
            if tab.editor.GetText() != txt and not self.notebook.GetPageText(page).startswith("*"):
                self.notebook.SetPageText(page, "*" + self.notebook.GetPageText(page))
            elif tab.editor.GetText() == txt and self.notebook.GetPageText(page).startswith("*"):
                self.notebook.SetPageText(page, self.notebook.GetPageText(page)[1:])
    #Filemenu Methods

    def new(self, message = "", path=""):
        self.new_file(self, message)

    def new_file(self, message = "", path=""):
        """Create a new empty tab"""
        self.add_file_tab("!New")
        tab = self.get_active_tab()
        if message != "":
            tab.editor.SetValue(message)
        if path != "":
            tab.path = path
            tab.name = os.path.split(path)[1]
            self.notebook.SetPageText(self.notebook.GetSelection(), tab.name)
            self.save_file()
        self.dplug.trigger_update("!New")
        self.lexer(self.dplug.base_config.get("startup", "proglang"))

    def open_file(self):
        """File dialog which passes the chosen file to 'editfl'"""
        tab = self.get_active_tab()
        if tab.isFile and tab.path != None:
            fp = os.path.split(tab.path)[0]
        else:
            fp = self.dplug.homedir
        dlg = wx.FileDialog(self, message="Open File",
                defaultDir=fp, defaultFile="", style=wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:   
            self.editfl(dlg.GetPath())
        dlg.Destroy()

    #editfl see notebook related methods
    
    def save(self):
        self.save_file()

    def save_file(self):
        """Save the current opened file to the location
            defined in the upper textctrl"""
        tab = self.get_active_tab()
        if tab.isFile and tab.name != "!New":
            try:
                txt = tab.editor.GetText()
                if not os.path.isdir(os.path.split(tab.path)[0]):   os.mkdir(os.path.split(tab.path)[0])
                f = open(tab.path, 'w')
                f.write(txt)
                f.close()
                svd = 'Saved: ' + tab.name + " @"+str(datetime.datetime.now())
                self.notebook.SetPageText(self.notebook.GetSelection(), tab.name)
                self.SetStatusText(svd)
                self.dplug.trigger_update("save")
            except:
                self.SetStatusText("Deditor was not able to save your file..")
        elif tab.isFile and tab.name == "!New":
            self.saveas()
        else:
            print "not a valid file tab"
    
    def saveas(self):
        tab = self.get_active_tab()
        dialog = wx.FileDialog( None, style = wx.SAVE | wx.OVERWRITE_PROMPT )
        if dialog.ShowModal() == wx.ID_OK:
            tab.path = dialog.GetPath()
            self.commandbar.SetValue(dialog.GetPath())
            self.save_file()
        dialog.Destroy()

    def reload_file(self):
        tab = self.get_active_tab()
        if tab.isFile and tab.name != "!New":
            try:
                f = open(tab.path, 'r')
                txt = ""
                for line in f.readlines():
                    txt = txt + line
                f.close()
                tab.editor.SetText(txt)
                self.dplug.trigger_update("open")
            except IOError:
                pass

    def quit(self, destroy = True):
        """Quit deditor"""
        self.dplug.trigger_update("quit")
        method = self.dplug.base_config.get("startup", "method")
        opn = self.dplug.base_config.get("startup","open")
        restore = []
        if method == "restore":
            for tab in self.tabs:
                if tab.path != None and os.path.isfile(tab.path):
                    restore.append(tab.path)
            self.dplug.base_config.set("startup", "focus", 
                self.notebook.GetSelection())
        if len(restore) > 0:
            restore = ",".join(restore)
        else:
            restore = ""
        if opn != restore:
            self.dplug.base_config.set("startup", "open", restore)
        if destroy: self.Destroy()

    #Viewmenu Methods

    def sh(self):
        """Show or hide the shell"""
        if self.shellview.IsChecked():
            self.console.Show()
            self.Layout()
        else:
            self.console.Hide()
            self.Layout()

    def config(self):
        """Show or Hide the configuration window"""
        if self.confview.IsChecked():
            self.confpan.Init()
            self.add_tab("Configuration", self.confpan)
        else:
            self.confpan = self.get_tab("Configuration").panel
            self.remove_tab("Configuration")
        self.Layout()

    def lexer(self, lex):
        self.get_active_tab().editor.SetLex(lex)

    #Findmenu Methods

    def find(self):
        """Show the find-bar"""
        if self.toolbar.IsShown():
            self.cfindbar()
        else:
            self.toolbar.Show()
            self.Layout()
            self.bartxt.SetFocus()

    def findbar(self):
        """locate a string in the current notebook page"""
        global lastsearch
        tab = self.get_active_tab()
        word = self.bartxt.GetValue()
        pos = int(tab.editor.FindText(
            0, tab.editor.GetTextLength(), word, self.get_find_mode()))
        if pos != -1:
            tab.editor.GotoPos(pos)
            tab.editor.SetSelection(pos, pos + len(word))
            lastsearch = pos + len(word)

    def cfindbar(self):
        """Close findbar"""
        self.toolbar.Hide()
        self.Layout()

    def findnext(self):
        """Find the next string"""
        global lastsearch
        tab = self.get_active_tab()
        start = lastsearch
        word = self.bartxt.GetValue()
        pos = int(tab.editor.FindText(start, 
            tab.editor.GetTextLength(),
            word, self.get_find_mode()))
        if pos != -1:
            tab.editor.GotoPos(pos+len(word))
            tab.editor.SetSelection(pos, pos + len(word))
            lastsearch = pos+len(word)
        if pos == -1:   self.findbar()

    def findprev(self):
        """Find the previous string"""
        global lastsearch #still call and set lastsearch to provide findnext the right information
        tab = self.get_active_tab()
        word = self.bartxt.GetValue()
        tab.editor.SearchAnchor()
        pos = tab.editor.SearchPrev(self.get_find_mode(), word)
        if pos != -1:
            tab.editor.GotoPos(pos)
            tab.editor.SetSelection(pos, pos + len(word))
            lastsearch = pos
        if pos == -1:   self.findbar()
    
    def get_find_mode(self):
        if self.mode == "WholeWord":
            return wx.stc.STC_FIND_WHOLEWORD
        elif self.mode == "Regex":
            return wx.stc.STC_FIND_REGEXP
        elif self.mode == "MatchCase":
            return wx.stc.STC_FIND_MATCHCASE
        elif self.mode == "WordStart":
            return wx.stc.STC_FIND_WORDSTART
        else:
            return 0

    def locate_line(self):
        """Goto line number"""
        line = IODialog("Goto Line Number", "Line number?").GetValue()
        if line:
            tab = self.get_active_tab()
            tab.editor.GotoLine(int(line))
    
    ########__Tabs__###########
    def add_file_tab(self, name, path = None):
        panel = wx.Panel(self.notebook, -1)
        sizer = wx.BoxSizer(wx.VERTICAL)
        editor = DemoCodeEditor(panel)
        editor.IndicatorSetStyle(2, wx.stc.STC_INDIC_SQUIGGLE)
        editor.IndicatorSetForeground(2, wx.RED)
        editor.StartStyling(1, wx.stc.STC_INDICS_MASK)
        editor.SetStyling(10, wx.stc.STC_INDIC2_MASK)
        sizer.Add(editor, -1, wx.EXPAND)
        panel.SetSizer(sizer)
        tab = Tab(name, path, panel, True)
        tab.editor = editor
        self.tabs.append(tab)
        self.notebook.AddPage(panel, name)
        self.notebook.SetSelection(self.notebook.GetPageCount() - 1)
    
    def add_tab(self, name, panel):
        self.tabs.append(Tab(name, None, panel))
        self.notebook.AddPage(panel, name)
        self.notebook.SetSelection(self.notebook.GetPageCount() - 1)

    def get_tab(self, name):
        found = []
        for tab in self.tabs:
            if tab.name == name:
                found.append(tab)
        if len(found) == 0:
            print "No tab with the name %s found" % name
        elif len(found) > 1:
            print "ERROR: there are more tabs with the name %s" % name
        else:
            return found[0]

    def close_tab(self, tab=None):
        """ Close the currently opened tab """
        if tab == None:
            page = self.notebook.GetSelection()
        else:
            page = int(tab)
        tab = self.tabs[page]
        if tab.isFile:
            if tab.name == "!New" and tab.editor.GetText() == "": pass
            elif tab.name == "!New":
                if CSDialog("Save Changes", "There are unsaved changes.\nDo you wish to save now?").GetValue():
                    self.save_file()
            else:
                f = open(tab.path, 'r')
                txt = ""
                for line in f.readlines():
                    txt = txt + line
                f.close()
                if tab.editor.GetText() != txt:
                    if CSDialog("Save Changes", "There are unsaved changes. Save?").GetValue():
                        self.save_file()
        elif tab.name == "Configuration" and self.confview.IsChecked():
            self.confview.Check(0)
        self.tabs.pop(page)
        self.notebook.RemovePage(page)
        if len(self.tabs) == 0:
            self.add_file_tab("!New", path="")

    def remove_tab(self, name):
        found = []
        for tab in self.tabs:
            if tab.name == name:
                found.append(tab)
        if len(found) == 0:
            print "No tab with the name %s found" % name
        elif len(found) > 1:
            print "ERROR: there are more tabs with the name %s; please close the tab(s) manually" % name
        else:
            i = self.tabs.index(found[0])
            self.tabs.remove(found[0])
            self.notebook.RemovePage(i)
    
    def get_active_tab(self):
        if self.notebook.GetPageCount() <= 0:
            self.new_file()
        page = self.notebook.GetSelection()
        return self.tabs[page]

class Tab():
    """Class to Add a new tab to the notebook"""
    def __init__(self, name, path = None, panel = None, isFile = False):
        self.panel = panel
        self.path = path
        self.name = name
        self.isFile = isFile
        if self.isFile:
            self.editor = None

class Config(wx.Panel):
    def __init__(self, dplug):
        wx.Panel.__init__(self, parent = dplug.main_class.notebook)
        self.dplug = dplug
        self.configsiz = wx.BoxSizer(wx.VERTICAL)
        self.top = wx.Panel(self)
        self.topsiz = wx.BoxSizer(wx.HORIZONTAL)
        self.confignav = wx.Panel(self.top)
        self.confignavsiz = wx.BoxSizer(wx.VERTICAL)
        self.navgen = wx.Button(self.confignav, -1, "General", style = wx.NO_BORDER)
        self.navkey = wx.Button(self.confignav, -1, "Keybindings", style = wx.NO_BORDER)
        self.navplg = wx.Button(self.confignav, -1, "Plugins", style = wx.NO_BORDER)
        self.confignavsiz.Add(self.navgen)
        self.confignavsiz.Add(self.navkey)
        self.confignavsiz.Add(self.navplg)
        self.confignav.SetSizer(self.confignavsiz)
        
        self.general = wx.Panel(self.top)
        self.generalsiz = wx.GridSizer(5,2,0,0)
        ##General / Startup
        self.load = wx.CheckBox( self.general, -1, "Load last session on startup")
        self.pyth = wx.CheckBox( self.general, -1, "Show Python Shell on startup")
        self.plugbox = wx.CheckBox( self.general, -1, "Enable plugins")
        self.design = wx.CheckBox( self.general, -1, "Extended interface")
        self.proglangs = ['ADA', 'APDL', 'ASM', 'ASN1', 'AU3', 'AUTOMATIC', 'AVE', 'BAAN', 'BASH', 'BATCH',
            'BLITZBASIC', 'BULLANT', 'CAML', 'CLW', 'CLWNOCASE', 'CONF', 'CONTAINER', 'CPP',
            'CPPNOCASE', 'CSOUND', 'CSS', 'DIFF', 'EIFFEL', 'EIFFELKW', 'ERLANG', 'ERRORLIST',
            'ESCRIPT', 'F77', 'FLAGSHIP', 'FORTH', 'FORTRAN', 'FREEBASIC', 'GUI4CLI', 'HASKELL',
            'HTML', 'INNOSETUP', 'KIX', 'LATEX', 'LISP', 'LOT', 'LOUT', 'LUA', 'MAKEFILE', 'MATLAB',
            'METAPOST', 'MMIXAL', 'MSSQL', 'NNCRONTAB', 'NSIS', 'NULL', 'OCTAVE', 'OPAL', 'PASCAL',
            'PERL', 'PHPSCRIPT', 'POV', 'POWERBASIC', 'PROPERTIES', 'PS', 'PUREBASIC', 'PYTHON',
            'REBOL', 'RUBY', 'SCRIPTOL', 'SMALLTALK', 'SPECMAN', 'SPICE', 'SQL', 'TADS3', 'TCL',
            'TEX', 'VB', 'VBSCRIPT', 'VERILOG', 'VHDL', 'XCODE', 'XML', 'YAML']
        self.prog = wx.Choice(self.general, choices=self.proglangs)
        self.generalsiz.Add(self.load)
        self.generalsiz.AddSpacer( ( 0, 0), 1, wx.EXPAND, 5 )
        self.generalsiz.Add(self.pyth)
        self.generalsiz.AddSpacer( ( 0, 0), 1, wx.EXPAND, 5 )
        self.generalsiz.Add(self.design)
        self.generalsiz.AddSpacer( ( 0, 0), 1, wx.EXPAND, 5 )
        self.generalsiz.Add(self.plugbox)
        self.generalsiz.AddSpacer( ( 0, 0), 1, wx.EXPAND, 5 )
        self.generalsiz.Add(wx.StaticText(self.general, -1, "Language to use by default"))
        self.generalsiz.Add(self.prog)
        self.general.SetSizer(self.generalsiz)
        #self.general.Hide()
        
        ##Keybindings
        self.keybindings = wx.Panel(self.top)
        self.keybindingssiz = wx.BoxSizer(wx.VERTICAL)
        self.keybook = wx.Notebook(self.keybindings)
        self.keylist = []
        self.keybindingssiz.Add(self.keybook, -1, wx.EXPAND)
        self.keybindings.SetSizer(self.keybindingssiz)
        
        ##Plugs
        self.plugins = wx.lib.scrolledpanel.ScrolledPanel(self.top)
        pluglist = self.dplug.GetPlugins()
        self.pluginssiz = wx.GridSizer(len(pluglist), 2, 0, 0)
        for plugin in pluglist:
            self.pluginssiz.Add(wx.StaticText(self.plugins, -1, plugin))
            x = wx.CheckBox(self.plugins)
            if plugin in [i.strip(" '") for i in self.dplug.plugin_config.get("general", "enabled").split(",")]: x.SetValue(1)
            self.pluginssiz.Add(x)
            self.Bind(wx.EVT_CHECKBOX, self.__plug_factory(plugin), x)
        self.plugins.SetSizer(self.pluginssiz)
        
        ###Conf-Notebook
        self.configsave = wx.Button(self, -1, "Save")
        self.dedrestart = wx.Button(self, -1, "Restart")
        self.funcsiz = wx.BoxSizer(wx.HORIZONTAL)
        self.funcsiz.Add(self.configsave)
        self.funcsiz.Add(wx.StaticText(self, -1, ''), -1, wx.EXPAND)
        self.funcsiz.Add(self.dedrestart)
        self.topsiz.Add(self.confignav, 0, wx.EXPAND)
        self.topsiz.Add(self.general)
        self.topsiz.Add(self.keybindings, -1, wx.EXPAND)
        self.topsiz.Add(self.plugins, -1, wx.EXPAND)
        self.keybindings.Hide()
        self.plugins.Hide()
        self.top.SetSizer(self.topsiz)
        self.configsiz.Add(self.top, -1, wx.EXPAND)
        self.configsiz.Add(self.funcsiz, 0, wx.EXPAND)
        self.SetSizer(self.configsiz)

        self.navitems = [["general", self.general], ["keybindings", self.keybindings], ["plugins", self.plugins]]
        self.Bind(wx.EVT_BUTTON, lambda event: self.navshow("general"), self.navgen)
        self.Bind(wx.EVT_BUTTON, lambda event: self.navshow("keybindings"), self.navkey)
        self.Bind(wx.EVT_BUTTON, lambda event: self.navshow("plugins"), self.navplg)
        self.Bind(wx.EVT_BUTTON, lambda event: self.save_config(), self.configsave)
        self.Bind(wx.EVT_BUTTON, lambda event: self.restart(), self.dedrestart)

        self.Hide()
    
    def navshow(self, item):
        self.navhide()
        for x in self.navitems:
            if x[0] == item:
                x[1].Show()
        self.Layout()
    
    def navhide(self):
        for x in self.navitems:
            x[1].Hide()
    
    def __plug_factory(self, i):
        """Switcher help function"""
        def _switcher(event):
            """helper function"""
            return self.PlugCheck(i)
        return _switcher

    def PlugCheck(self, plug):
        e = self.dplug.plugin_config.get("general", "enabled").split(",")
        d = self.dplug.plugin_config.get("general", "disabled").split(",")
        print e, d
        if plug in e:
            e.remove(plug)
            d.append(plug)
            self.dplug.plugin_config.set("general", "disabled", ",".join(d))
            self.dplug.plugin_config.set("general", "enabled", ",".join(e))
            if CSDialog("Plugins Configuration", "Deditor will need to restart before this is changed.\nRestart now?").GetValue():
                self.dplug.main_class.quit(False)
                python = sys.executable
                os.execl(python, python, * sys.argv)
        else:
            d.remove(plug)
            e.append(plug)
            try:
                self.dplug.register(plug)
                self.dplug.run("init_dplug", plug, self.dplug)
            except AttributeError:
                print "Error activating " + plug
            self.dplug.plugin_config.set("general", "disabled", ",".join(d))
            self.dplug.plugin_config.set("general", "enabled", ",".join(e))
        self.Layout()

    def restart(self):
        self.dplug.main_class.quit(False)
        python = sys.executable
        os.execl(python, python, * sys.argv)

    def enable_plugin(self):
        if self.plugbox.IsChecked():
            self.dplug.plugin_config.set("general", "allowed", "True")
            self.dplug.Init_Plugins()
        else:
            self.dplug.plugin_config.set("general", "allowed", "False")
            if CSDialog("Plugins Configuration", "Deditor will need to restart before this is changed.\nRestart now?").GetValue():
                self.dplug.main.quit(False)
                python = sys.executable
                os.execl(python, python, * sys.argv)

    def save_config(self):
        """Save the analyze settings in the configuration window"""
        #General Tab
        start = self.load.GetValue()
        if start == 0:
            startup = "!New"
            opn = ""
        elif start == 1:
            startup = "restore"
            opn = ""
        self.dplug.base_config.set("startup","open",opn)
        self.dplug.base_config.set("startup","focus",0)
        self.dplug.base_config.set("startup","method",startup)
        if self.pyth.IsChecked() == True:   self.dplug.base_config.set("startup", "python", "True")
        else:   self.dplug.base_config.set("startup", "python", "False")
        self.dplug.base_config.set("startup", "proglang", self.proglangs[self.prog.GetSelection()])
        #KeyBindings Tab
        newlist =[]
        for x in self.keylist:
            try:
                newlist.append([x[0],x[1],x[2].GetValue()])
            except:
                newlist.append([x[0],x[1], self.dplug.get_keybinding(x[0], x[1])])
        self.dplug.save_bindings(newlist)
        #Plugin enable
        if self.plugbox.IsChecked(): self.dplug.plugin_config.set("general","allowed","True")
        else: self.dplug.plugin_config.set("general","allowed","False")
        #Plugins
        self.dplug.trigger_update("configsave")
        #general
        self.dplug.base_config.set("general","design", ["minimal","extend"][self.design.GetValue()])
        
    def Init(self):
        self.keybook.DeleteAllPages()
        for project in self.dplug.get_project_bindings():
            temp_page = wx.lib.scrolledpanel.ScrolledPanel(self.keybook)
            sgrid = wx.GridSizer(len(project[1]),2,0,0)
            for binding in project[1::]:
                i = wx.StaticText(temp_page,-1,binding[0])
                j = wx.TextCtrl(temp_page, -1, binding[1])
                box = wx.BoxSizer(wx.VERTICAL)
                box.Add(j, 0, wx.EXPAND)
                sgrid.Add(i)
                sgrid.Add(box, -1, wx.EXPAND)
                self.keylist.append([project[0],binding[0],j])
            temp_page.SetupScrolling()
            temp_page.SetSizer(sgrid)
            self.keybook.AddPage(temp_page, project[0])
        #Startup Prefs
        if self.dplug.base_config.get("startup","method") == "restore":    self.load.SetValue(1)
        else:   self.load.SetValue(0)
        if self.dplug.base_config.get("startup", "python") == "True":  self.pyth.SetValue(1)
        else:   self.pyth.SetValue(0)
        self.prog.SetSelection(self.proglangs.index(self.dplug.base_config.get("startup", "proglang")))
        self.design.SetValue(["minimal","extend"].index(self.dplug.base_config.get("general", "design")))
        #Plugin enable
        if self.dplug.plugin_config.get("general","allowed") == "True": self.plugbox.SetValue(1)
        else: self.plugbox.SetValue(0)
        #Plugins
        self.dplug.trigger_update("configinit")
    
    def plugin_add(self, panel, expand = True):
        if expand:
            self.topsiz.Add(panel, -1, wx.EXPAND)
        else:
            self.topsiz.Add(panel)
        panel.Hide()
        temp_button = wx.Button(self.confignav, -1, panel.button_name, style = wx.NO_BORDER)
        self.confignavsiz.Add(temp_button)
        self.navitems.append([panel.button_name, panel])
        self.Bind(wx.EVT_BUTTON, lambda event: self.navshow(panel.button_name), temp_button)
        

if __name__ == "__main__":
    com = Com()
    app = wx.PySimpleApp(0)
    wx.InitAllImageHandlers()

    com.register("deditor")
    com.SetMainModule("deditor")
    com.InitMainClass("DEDITOR")
    
    com.Init_Plugins()
    
    #com.Update()
    
    app.SetTopWindow(com.main_class)
    com.main_class.Show()

    app.MainLoop()
